unit CurEdit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ColorGrd, ExtCtrls, StdCtrls, ComCtrls, RegEdit;

type
 TCursorDir=packed record
  cdReserved :word;
  cdType     :word;
  cdCount    :word;
  //cdEntries  :array[0..cdCount-1] of TCursorDirEntry;
 end;

 TCursorDirEntry=packed record
  bWidth       :byte;
  bHeight      :byte;
  bColorCount  :byte;
  bReserved    :byte;
  wXHotspot    :word;
  wYHotspot    :word;
  IBytesInRes  :integer;
  dwImageOffset:longint;
  //Colors       :array[0..1 SHL bColorCount] of TColor;
  //Bitmap       :array[0..31,0..31] of ColorBits;
  //ANDMask      :array[0..31] of integer;
 end;

 TCursorData=class;

 TColors=array[0..255] of TRGBQUAD;
 Bytes  =array[0..MaxInt-1] of byte;

 TCursorEntry=class
 private
  Owner :TCursorData;
  Entry :TCursorDirEntry;
  Header:TBitmapInfoHeader;
  Colors:^TColors;
  ColorCount:integer;
  Bitmap:^Bytes;
  Mask  :^Bytes;
 public
  Constructor Create(AOwner:TCursorData);
  Destructor  Destroy;
  procedure LoadFromStream(AStream:TStream);
  procedure SaveToStream1(AStream:TStream; Var Size:integer);
  procedure SaveToStream2(AStream:TStream);
  procedure Draw(ACanvas:TCanvas);
  function GetColor(Index:integer):TColor;
  procedure plot(x,y,c,m:integer);
 end;

 TCursorData=class
 private
  Dir    :TCursorDir;
  Entries:TList;
 public
  Constructor Create;
  Destructor  Destroy;
  procedure Clear;
  procedure LoadFromFile(AFileName:string);
  procedure SaveToFile(Index:integer;AFileName:string);
  procedure LoadFromStream(AStream:TStream);
  procedure SaveToStream(Index:integer; AStream:TStream);
  function Count:integer;
  function Entry(Index:integer):TCursorEntry;
  procedure Draw(Index:integer; ACanvas:TCanvas);
  function Hotspot(index:integer):TPoint;
  function Bitmap:pointer;
  function Mask:pointer;
 end;

  TForm1 = class(TForm)
    Image1: TImage;
    Image2: TImage;
    mTrans: TShape;
    mXOR: TShape;
    Hspot: TPaintBox;
    VSpot: TPaintBox;
    Button1: TButton;
    Button2: TButton;
    ShowGrid: TCheckBox;
    Navigate: TUpDown;
    Label2: TLabel;
    Palette: TImage;
    procedure FormCreate(Sender: TObject);
    procedure Image1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure HspotPaint(Sender: TObject);
    procedure VSpotPaint(Sender: TObject);
    procedure HspotMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure VSpotMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Image1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormDestroy(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure ShowGridClick(Sender: TObject);
    procedure NavigateClick(Sender: TObject; Button: TUDBtnType);
    procedure mTransMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PaletteMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Image2MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure Image2MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Image2MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    { Dclarations prives }
    Data:TCursorData;
    Zoom:integer;
    Index:integer;
    ClickX:integer;
    ClickY:integer;
    AColor:integer;
    hCursor:HCursor;
    OldCursor:HCursor;
    iColor:integer;
    Mask:integer;
    IsDown:boolean;
    Edited:boolean;
    procedure SetIndex(Value:integer);
    procedure SetZoom(Value:integer);
    procedure Plot(x,y:integer;Hotspot:boolean);
    procedure SetMask(Value:integer);
    procedure SetColor(Value:integer);
  public
    { Dclarations publiques }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

const
 k=16;

//---------------------------------------------------------------------------//

Constructor TCursorEntry.Create(AOwner:TCursorData);
 begin
  Owner:=AOwner;
 end;

Destructor TCursorEntry.Destroy;
 begin
  FreeMem(Colors);
  FreeMem(Bitmap);
  FreeMem(Mask);
  inherited;
 end;

procedure TCursorEntry.LoadFromStream(AStream:TStream);
 var
  Index:integer;
  Size :integer;
 begin
  AStream.ReadBuffer(Entry,SizeOf(Entry));
  index:=AStream.Position;

  AStream.Position:=Entry.dwImageOffset;
  AStream.ReadBuffer(Header,SizeOf(Header));

  ColorCount:=1 SHL (Header.biBitCount);
  Size:=ColorCount*SizeOf(TColor);
  GetMem(Colors,Size);
  AStream.ReadBuffer(Colors^,Size);

  Size:=Entry.bWidth*Entry.bHeight*Header.biBitCount div 8;
  GetMem(Bitmap,Size);
  AStream.ReadBuffer(Bitmap^,Size);

  Size:=Entry.bWidth*Entry.bHeight div 8;
  GetMem(Mask  ,Size);
  AStream.ReadBuffer(Mask^  ,Size);

  AStream.Position:=index;
 end;

procedure TCursorEntry.SaveToStream1(AStream:TStream;Var Size:integer);
 begin
  Entry.dwImageOffset:=Size;
  Inc(Size,
   SizeOf(Header)+
   ColorCount*SizeOf(TColor)+
   Entry.bWidth*Entry.bHeight*Header.biBitCount div 8+
   Entry.bWidth*Entry.bHeight div 8
  );
  AStream.WriteBuffer(Entry,SizeOf(Entry));
 end;

procedure TCursorEntry.SaveToStream2(AStream:TStream);
 var
  Size :integer;
 begin
  AStream.WriteBuffer(Header,SizeOf(Header));

  Size:=(1 SHL (Header.biBitCount))*SizeOf(TColor);
  AStream.WriteBuffer(Colors^,Size);

  Size:=Entry.bWidth*Entry.bHeight*Header.biBitCount div 8;
  AStream.WriteBuffer(Bitmap^,Size);

  Size:=Entry.bWidth*Entry.bHeight div 8;
  AStream.WriteBuffer(Mask^  ,Size);
 end;

procedure TCursorEntry.Draw(ACanvas:TCanvas);
 var
  S,M:integer;
  x,y,z:integer;
  a:boolean;
  p:byte;
  cl:TColor;
 begin
  S:=8-Header.biBitCount;
  M:=ColorCount-1;
  with ACanvas do
   for y:=0 to 31 do
    for x:=0 to 31 do begin

     z:=32*(31-y)+x;
     a:=(Mask[z div 8] and (1 shl (7-(x and 7)))<>0);

     z:=Header.biBitCount*z;
     p:=Bitmap[z div 8];
     p:=p shr (S-(z mod 8));
     p:=p and M;

     if a then begin
      if p=0 then cl:=clOlive else cl:=clGray;
     end else begin
      cl:=GetColor(p);
     end;

     Pixels[x,y]:=cl;
    end;
 end;

Function TCursorEntry.GetColor(index:integer):TColor;
 begin
  with Colors[Index] do Result:=RGB(rgbRed,rgbGreen,rgbBlue);
 end;

procedure TCursorEntry.plot(x,y,c,m:integer);
 var
  z:integer;
  bm:byte;
 begin
  z:=32*(31-y)+x;
  bm:=1 shl (7-(x and 7));
  if m=0 then
   Mask[z div 8]:=Mask[z div 8] and (255 - bm)
  else
   Mask[z div 8]:=Mask[z div 8] or bm;

  z:=z*Header.biBitCount;
  bm:=(8-Header.biBitcount)-(z mod 8);
  c:=c shl (bm);
  bm:=NOT (( (1 shl Header.biBitCount)-1 ) shl bm);
  Bitmap[z div 8]:=(Bitmap[z div 8] and bm) or c;
 end;

//---------------------------------------------------------------------------//

Constructor TCursorData.Create;
 begin
  inherited;
  Entries:=TList.Create;
 end;

Destructor TCursorData.Destroy;
 begin
  Clear;
  Entries.Free;
  inherited;
 end;

procedure TCursorData.Clear;
 var
  i:integer;
 begin
  for i:=Entries.Count-1 downto 0 do TCursorEntry(Entries[i]).Free;
  Entries.Clear;
 end;

procedure TCursorData.LoadFromFile(AFileName:string);
 var
  fs:TFileStream;
 begin
  fs:=TFileStream.Create(AFileName,fmOpenRead);
  try
   LoadFromStream(fs);
  finally
   fs.Free;
  end;
 end;

procedure TCursorData.SaveToFile(Index:integer;AFileName:string);
 var
  fs:TFileStream;
 begin
  fs:=TFileStream.Create(AFileName,fmOpenWrite or fmCreate);
  try
   SaveToStream(Index,fs);
  finally
   fs.Free;
  end;
 end;

procedure TCursorData.LoadFromStream(AStream:TStream);
 var
  i:integer;
  Entry:TCursorEntry;
 begin
  AStream.ReadBuffer(Dir,SizeOf(Dir));
  for i:=0 to Dir.cdCount-1 do begin
   Entry:=TCursorEntry.Create(Self);
   Entry.LoadFromStream(AStream);
   Entries.Add(Entry);
  end;
 end;

procedure TCursorData.SaveToStream(Index:integer;AStream:TStream);
 var
  i:integer;
  Size:integer;
 begin
  if Index<0 then begin
   Dir.cdCount:=Entries.Count;
   AStream.WriteBuffer(Dir,SizeOf(Dir));
   Size:=SizeOf(Dir)+Entries.Count*SizeOf(TCursorDirEntry);
   for i:=0 to Entries.Count-1 do TCursorEntry(Entries[i]).SaveToStream1(AStream,Size);
   for i:=0 to Entries.Count-1 do TCursorEntry(Entries[i]).SaveToStream2(AStream);
  end else begin
   Dir.cdCount:=1;
   AStream.WriteBuffer(Dir,SizeOf(Dir));
   Size:=SizeOf(Dir)+SizeOf(TCursorDirEntry);
   TCursorEntry(Entries[Index]).SaveToStream1(AStream,Size);
   TCursorEntry(Entries[Index]).SaveToStream2(AStream);
  end;
 end;

function TCursorData.Count:integer;
 begin
  Result:=Entries.Count;
 end;

function TCursorData.Entry(Index:integer):TCursorEntry;
 begin
  Result:=Entries[Index];
 end;

procedure TCursorData.Draw(Index:integer; ACanvas:TCanvas);
 begin
  TCursorEntry(Entries[Index]).Draw(ACanvas);
 end;

function TCursorData.Hotspot(index:integer):TPoint;
 begin
  with TCursorEntry(Entries[Index]) do begin
   Result.X:=Entry.wXHotspot;
   Result.Y:=Entry.wYHotspot;
  end;
 end;

function TCursorData.Bitmap:pointer;
 begin
  Result:=TCursorEntry(Entries[0]).Bitmap;
 end;

function TCursorData.Mask:pointer;
 begin
  Result:=TCursorEntry(Entries[0]).Mask
 end;

//-----------------------
procedure TForm1.FormCreate(Sender: TObject);
var
 f:file;
 i:integer;
begin
// Register CUR files
 if ReadRegString(HKEY_CLASSES_ROOT,'.cur','','')='' then begin
  if Application.MessageBox(PChar('Associate CUR files to '+Application.ExeName+' ?'),'Confirmation',MB_YESNO)=idYES then begin
   WriteRegString(HKEY_CLASSES_ROOT,'.cur','','curfile');
   WriteRegString(HKEY_CLASSES_ROOT,'curfile\DefaultIcon','','%1');
   WriteRegString(HKEY_CLASSES_ROOT,'curfile\shell\open\command','',Application.exeName+' %1');
  end;
 end;

 if Paramstr(1)='' then halt;
 Caption:=ParamStr(1);
 Data:=TCursorData.Create;
 Data.LoadFromFile(ParamStr(1));
 Navigate.Max:=Data.Count-1;
 Image1.Picture.Bitmap.width:=Image1.Width;
 Image1.Picture.Bitmap.height:=Image1.Height;
 Zoom:=10;
 SetIndex(0);
 SetMask(0);
 iColor:=1;
 SetColor(0);
end;

procedure TForm1.SetIndex(Value:integer);
var
 x,y:integer;
begin
 Index:=Value;
 Label2.Caption:=IntToStr(Index+1)+'/'+IntToStr(Data.Count);
 Data.Draw(Index,Image1.Picture.Bitmap.Canvas);
 with Data.Hotspot(Index) do begin
  ClickX:=X;
  clickY:=Y;
 end;
 SetZoom(Zoom);
 with Data.Entry(Index) do begin
  with Palette.Picture.Bitmap do begin
   Width :=Self.Palette.Width;
   Height:=1+k*ColorCount div 2;
   with Canvas do begin
    Pen.Color:=clBlack;
    for y:=0 to ColorCount div 2 do
     for x:=0 to 1 do begin
      Brush.color:=GetColor(x+2*y);//with Colors[2*y+x] do Brush.Color:=RGB(rgbRed,rgbGreen,rgbBlue);
      Rectangle(k*x,k*y,k*x+k+1,k*y+k+1);
     end;
   end;
  end;
 end;
end;

procedure TForm1.SetZoom(Value:integer);
 var
  i:integer;
 begin
 Zoom:=Value;
 HSpot.SetBounds(HSpot.Left,Image2.Top-Zoom,Zoom*Image1.Width,Zoom);
 VSpot.SetBounds(Image2.Left-Zoom,VSpot.Top,Zoom,Zoom*Image1.Height);
 Image2.SetBounds(Image2.Left,Image2.Top,HSpot.Width,VSpot.Height);

 Image2.Picture.Bitmap.width :=Image2.Width;
 Image2.Picture.Bitmap.height:=Image2.Height;

 ShowGridClick(Self);
 SetColor(0);
end;

procedure TForm1.Image1MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
// hCursor:=CreateCursor(hInstance,ClickX,ClickY,32,32,Data.Bitmap,Data.Mask);
 Data.SaveToFile(Index,'CUREDIT.CUR');
 hCursor:=LoadCursorFromFile('CUREDIT.CUR'); // Support 16 colors Cursor (Unlike CreateCursor)...
 OldCursor:=SetCursor(hCursor);
 SetCaptureControl(Image1);
end;

procedure TForm1.HspotPaint(Sender: TObject);
var
 x:integer;
 p:array[0..2] of TPoint;
begin
 x:=Zoom*ClickX+Zoom div 2;
 with HSpot.Canvas do begin
  Brush.Color:=clWhite;
  FillRect(HSpot.ClientRect);
  p[0]:=Point(x-Zoom div 2,1);
  p[1]:=Point(x+Zoom div 2,1);
  p[2]:=Point(x,Zoom div 2+1);
  Brush.Color:=clBlack;
  Pen.Color:=clBlack;
  Polygon(p);
 end;
end;

procedure TForm1.VSpotPaint(Sender: TObject);
var
 y:integer;
 p:array[0..2] of TPoint;
begin
 y:=Zoom div 2+Zoom*ClickY;
 with VSpot.Canvas do begin
  Brush.Color:=clWhite;
  FillRect(VSpot.ClientRect);
  p[0]:=Point(1,y-Zoom div 2);
  p[1]:=Point(1,y+Zoom div 2);
  p[2]:=Point(Zoom div 2+1,y);
  Brush.Color:=clBlack;
  Pen.Color:=clBlack;
  Polygon(p);
 end;
end;

procedure TForm1.HspotMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
 Plot(ClickX,ClickY,False);
 ClickX:=x div Zoom;
 Plot(ClickX,ClickY,True);
 HSpotPaint(Self);
end;

procedure TForm1.VSpotMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
 Plot(ClickX,ClickY,False);
 ClickY:=y div Zoom;
 Plot(ClickX,ClickY,True);
 VSpotPaint(Self);
end;

procedure TForm1.Image1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
 SetCursor(OldCursor);
 DeleteObject(hCursor);
 SetCaptureControl(nil);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 Data.Free;
end;

procedure TForm1.Plot(x,y:integer;Hotspot:boolean);
 begin
  with Image2.canvas do begin
   Brush.Color:=Image1.Picture.bitmap.Canvas.Pixels[x,y];
   if Hotspot then Pen.Color:=clWhite else Pen.Color:=clBlack;
   Rectangle(Zoom*x,Zoom*y,Zoom*x+Zoom+1,Zoom*y+Zoom+1);
  end;
 end;

procedure TForm1.Button2Click(Sender: TObject);
var
 z:integer;
begin
 z:=Zoom+TButton(Sender).Tag;
 if (z>0)and(z<=10) then SetZoom(z);
end;

procedure TForm1.ShowGridClick(Sender: TObject);
var
 i:integer;
begin
 with Image2.Picture.Bitmap.Canvas do begin
  StretchDraw(Image2.ClientRect,Image1.Picture.Bitmap);
  if ShowGrid.Checked then begin
   Pen.Color:=clBlack;
   for i:=0 to 32 do begin
    MoveTo(Zoom*i,0); LineTo(Zoom*i,Image2.Height);
    MoveTo(0,Zoom*i); LineTo(Image2.Width,Zoom*i);
   end;
   Plot(ClickX,ClickY,True);
  end;
 end;
end;

procedure TForm1.NavigateClick(Sender: TObject; Button: TUDBtnType);
begin
 SetIndex(Navigate.Position);
end;

procedure TForm1.mTransMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 SetMask(TComponent(Sender).Tag);
end;

procedure TForm1.SetMask(Value:integer);
 begin
  Mask:=Value;
  case Mask of
   0:begin
      mTrans.Pen.color:=clWhite;
      mXOR.Pen.Color:=clBlack;
     end;
   1:begin
      mTrans.Pen.Color:=clBlack;
      mXOR.Pen.Color:=clWhite;
     end;
  end;
 end;

procedure TForm1.PaletteMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 SetColor((x div k)+2*(y div k));
end;

procedure TForm1.SetColor(Value:integer);
 var
  x,y:integer;
 begin
  if iColor=Value then exit;
  with Palette.Picture.Bitmap.Canvas do begin
   Brush.Style:=bsClear;
   Pen.Color:=clBlack;
   x:=iColor mod 2;
   y:=iColor div 2;
   Rectangle(k*x,k*y,k*x+k+1,k*y+k+1);
   Pen.Color:=clWhite;
   x:=Value mod 2;
   y:=Value div 2;
   Rectangle(k*x,k*y,k*x+k+1,k*y+k+1);
  end;
  iColor:=Value;
 end;

procedure TForm1.Image2MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var
 cl:TColor;
begin
 if Isdown then begin
  x:=x div zoom;
  y:=y div zoom;

  if Mask=1 then begin
   if iColor=0 then cl:=clOlive else cl:=clGray;
  end else begin
   cl:=Data.Entry(Index).GetColor(iColor);
  end;

  Image1.Picture.bitmap.Canvas.Pixels[x,y]:=cl;
  with Image2.Picture.Bitmap.Canvas do begin
   Brush.Color:=cl;
   Pen.Color:=cl;
   Rectangle(Zoom*x+1,Zoom*y+1,Zoom*x+Zoom,Zoom*y+Zoom);
   Data.Entry(Index).Plot(x,y,iColor,Mask);
  end;
 end;
end;

procedure TForm1.Image2MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
 IsDown:=True;
 Edited:=True;
 Image2MouseMove(Sender,Shift,X,Y);
end;

procedure TForm1.Image2MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
 Isdown:=false;
end;

procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
 if Edited then begin
  case Application.MessageBox(PChar('Save change in '+Caption+' ?'),'Confirmation',MB_YESNOCANCEL or MB_ICONQUESTION) of
   idYES:Data.SaveToFile(-1,Caption);
   idNO :CanClose:=True;
   else  CanClose:=False;
  end;
 end;
end;

end.
